@@ -2,10 +2,10 @@
 /*
 ################################################################################
 #
-# r8101 is the Linux device driver released for Realtek Fast Ethernet
+# r8125 is the Linux device driver released for Realtek 2.5 Gigabit Ethernet
 # controllers with PCI-Express interface.
 #
-# Copyright(c) 2022 Realtek Semiconductor Corp. All rights reserved.
+# Copyright(c) 2024 Realtek Semiconductor Corp. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the Free
@@ -36,48 +36,50 @@
 #include <linux/version.h>
 #include <linux/pci.h>
 #include <linux/netdevice.h>
+#include <linux/delay.h>
+#include <linux/in.h>
+#include <linux/ethtool.h>
 #include <asm/uaccess.h>
-#include "r8101.h"
+#include "r8125.h"
+#include "rtl_eeprom.h"
 #include "rtltool.h"
 
-int rtl8101_tool_ioctl(struct rtl8101_private *tp, struct ifreq *ifr)
+int rtl8125_tool_ioctl(struct rtl8125_private *tp, struct ifreq *ifr)
 {
-        struct rtltool_cmd	my_cmd;
-        unsigned long flags;
-        int	ret;
+        struct rtltool_cmd my_cmd;
+        int ret;
 
-        if (copy_from_user(&my_cmd, ifr->ifr_data, sizeof(struct rtltool_cmd))) {
+        if (copy_from_user(&my_cmd, ifr->ifr_data, sizeof(my_cmd)))
                 return -EFAULT;
-        }
 
         ret = 0;
-        switch(my_cmd.cmd) {
+        switch (my_cmd.cmd) {
         case RTLTOOL_READ_MAC:
-                if(my_cmd.len==1) {
+                if (my_cmd.len==1)
                         my_cmd.data = readb(tp->mmio_addr+my_cmd.offset);
-                } else if(my_cmd.len==2) {
+                else if (my_cmd.len==2)
                         my_cmd.data = readw(tp->mmio_addr+(my_cmd.offset&~1));
-                } else if(my_cmd.len==4) {
+                else if (my_cmd.len==4)
                         my_cmd.data = readl(tp->mmio_addr+(my_cmd.offset&~3));
-                } else {
+                else {
                         ret = -EOPNOTSUPP;
                         break;
                 }
 
-                if (copy_to_user(ifr->ifr_data, &my_cmd, sizeof(struct rtltool_cmd))) {
+                if (copy_to_user(ifr->ifr_data, &my_cmd, sizeof(my_cmd))) {
                         ret = -EFAULT;
                         break;
                 }
                 break;
 
         case RTLTOOL_WRITE_MAC:
-                if(my_cmd.len==1) {
+                if (my_cmd.len==1)
                         writeb(my_cmd.data, tp->mmio_addr+my_cmd.offset);
-                } else if(my_cmd.len==2) {
+                else if (my_cmd.len==2)
                         writew(my_cmd.data, tp->mmio_addr+(my_cmd.offset&~1));
-                } else if(my_cmd.len==4) {
+                else if (my_cmd.len==4)
                         writel(my_cmd.data, tp->mmio_addr+(my_cmd.offset&~3));
-                } else {
+                else {
                         ret = -EOPNOTSUPP;
                         break;
                 }
@@ -85,11 +87,8 @@ int rtl8101_tool_ioctl(struct rtl8101_private *tp, struct ifreq *ifr)
                 break;
 
         case RTLTOOL_READ_PHY:
-                spin_lock_irqsave(&tp->lock, flags);
-                my_cmd.data = rtl8101_mdio_prot_read(tp, my_cmd.offset);
-                spin_unlock_irqrestore(&tp->lock, flags);
-
-                if (copy_to_user(ifr->ifr_data, &my_cmd, sizeof(struct rtltool_cmd))) {
+                my_cmd.data = rtl8125_mdio_prot_read(tp, my_cmd.offset);
+                if (copy_to_user(ifr->ifr_data, &my_cmd, sizeof(my_cmd))) {
                         ret = -EFAULT;
                         break;
                 }
@@ -97,17 +96,12 @@ int rtl8101_tool_ioctl(struct rtl8101_private *tp, struct ifreq *ifr)
                 break;
 
         case RTLTOOL_WRITE_PHY:
-                spin_lock_irqsave(&tp->lock, flags);
-                rtl8101_mdio_prot_write(tp, my_cmd.offset, my_cmd.data);
-                spin_unlock_irqrestore(&tp->lock, flags);
+                rtl8125_mdio_prot_write(tp, my_cmd.offset, my_cmd.data);
                 break;
 
         case RTLTOOL_READ_EPHY:
-                spin_lock_irqsave(&tp->lock, flags);
-                my_cmd.data = rtl8101_ephy_read(tp, my_cmd.offset);
-                spin_unlock_irqrestore(&tp->lock, flags);
-
-                if (copy_to_user(ifr->ifr_data, &my_cmd, sizeof(struct rtltool_cmd))) {
+                my_cmd.data = rtl8125_ephy_read(tp, my_cmd.offset);
+                if (copy_to_user(ifr->ifr_data, &my_cmd, sizeof(my_cmd))) {
                         ret = -EFAULT;
                         break;
                 }
@@ -115,17 +109,13 @@ int rtl8101_tool_ioctl(struct rtl8101_private *tp, struct ifreq *ifr)
                 break;
 
         case RTLTOOL_WRITE_EPHY:
-                spin_lock_irqsave(&tp->lock, flags);
-                rtl8101_ephy_write(tp, my_cmd.offset, my_cmd.data);
-                spin_unlock_irqrestore(&tp->lock, flags);
+                rtl8125_ephy_write(tp, my_cmd.offset, my_cmd.data);
                 break;
 
         case RTLTOOL_READ_ERI:
                 my_cmd.data = 0;
                 if (my_cmd.len==1 || my_cmd.len==2 || my_cmd.len==4) {
-                        spin_lock_irqsave(&tp->lock, flags);
-                        my_cmd.data = rtl8101_eri_read(tp, my_cmd.offset, my_cmd.len, ERIAR_ExGMAC);
-                        spin_unlock_irqrestore(&tp->lock, flags);
+                        my_cmd.data = rtl8125_eri_read(tp, my_cmd.offset, my_cmd.len, ERIAR_ExGMAC);
                 } else {
                         ret = -EOPNOTSUPP;
                         break;
@@ -140,9 +130,7 @@ int rtl8101_tool_ioctl(struct rtl8101_private *tp, struct ifreq *ifr)
 
         case RTLTOOL_WRITE_ERI:
                 if (my_cmd.len==1 || my_cmd.len==2 || my_cmd.len==4) {
-                        spin_lock_irqsave(&tp->lock, flags);
-                        my_cmd.data = rtl8101_eri_write(tp, my_cmd.offset, my_cmd.len, my_cmd.data, ERIAR_ExGMAC);
-                        spin_unlock_irqrestore(&tp->lock, flags);
+                        rtl8125_eri_write(tp, my_cmd.offset, my_cmd.len, my_cmd.data, ERIAR_ExGMAC);
                 } else {
                         ret = -EOPNOTSUPP;
                         break;
@@ -151,61 +139,92 @@ int rtl8101_tool_ioctl(struct rtl8101_private *tp, struct ifreq *ifr)
 
         case RTLTOOL_READ_PCI:
                 my_cmd.data = 0;
-                if(my_cmd.len==1) {
-                        pci_read_config_byte(tp->pci_dev, my_cmd.offset, (u8 *)&my_cmd.data);
-                } else if(my_cmd.len==2) {
-                        pci_read_config_word(tp->pci_dev, my_cmd.offset, (u16 *)&my_cmd.data);
-                } else if(my_cmd.len==4) {
-                        pci_read_config_dword(tp->pci_dev, my_cmd.offset, &my_cmd.data);
-                } else {
+                if (my_cmd.len==1)
+                        pci_read_config_byte(tp->pci_dev, my_cmd.offset,
+                                             (u8 *)&my_cmd.data);
+                else if (my_cmd.len==2)
+                        pci_read_config_word(tp->pci_dev, my_cmd.offset,
+                                             (u16 *)&my_cmd.data);
+                else if (my_cmd.len==4)
+                        pci_read_config_dword(tp->pci_dev, my_cmd.offset,
+                                              &my_cmd.data);
+                else {
                         ret = -EOPNOTSUPP;
                         break;
                 }
 
-                if (copy_to_user(ifr->ifr_data, &my_cmd, sizeof(struct rtltool_cmd))) {
+                if (copy_to_user(ifr->ifr_data, &my_cmd, sizeof(my_cmd))) {
                         ret = -EFAULT;
                         break;
                 }
                 break;
 
         case RTLTOOL_WRITE_PCI:
-                if(my_cmd.len==1) {
-                        pci_write_config_byte(tp->pci_dev, my_cmd.offset, my_cmd.data);
-                } else if(my_cmd.len==2) {
-                        pci_write_config_word(tp->pci_dev, my_cmd.offset, my_cmd.data);
-                } else if(my_cmd.len==4) {
-                        pci_write_config_dword(tp->pci_dev, my_cmd.offset, my_cmd.data);
-                } else {
+                if (my_cmd.len==1)
+                        pci_write_config_byte(tp->pci_dev, my_cmd.offset,
+                                              my_cmd.data);
+                else if (my_cmd.len==2)
+                        pci_write_config_word(tp->pci_dev, my_cmd.offset,
+                                              my_cmd.data);
+                else if (my_cmd.len==4)
+                        pci_write_config_dword(tp->pci_dev, my_cmd.offset,
+                                               my_cmd.data);
+                else {
                         ret = -EOPNOTSUPP;
                         break;
                 }
 
                 break;
 
-        case RTL_ENABLE_PCI_DIAG:
-                spin_lock_irqsave(&tp->lock, flags);;
-                tp->rtk_enable_diag = 1;
-                spin_unlock_irqrestore(&tp->lock, flags);
+        case RTLTOOL_READ_EEPROM:
+                my_cmd.data = rtl8125_eeprom_read_sc(tp, my_cmd.offset);
+                if (copy_to_user(ifr->ifr_data, &my_cmd, sizeof(my_cmd))) {
+                        ret = -EFAULT;
+                        break;
+                }
 
-                printk("enable rtk diag\n");
+                break;
+
+        case RTLTOOL_WRITE_EEPROM:
+                rtl8125_eeprom_write_sc(tp, my_cmd.offset, my_cmd.data);
+                break;
+
+        case RTL_READ_OOB_MAC:
+                rtl8125_oob_mutex_lock(tp);
+                my_cmd.data = rtl8125_ocp_read(tp, my_cmd.offset, 4);
+                rtl8125_oob_mutex_unlock(tp);
+                if (copy_to_user(ifr->ifr_data, &my_cmd, sizeof(my_cmd))) {
+                        ret = -EFAULT;
+                        break;
+                }
+                break;
+
+        case RTL_WRITE_OOB_MAC:
+                if (my_cmd.len == 0 || my_cmd.len > 4)
+                        return -EOPNOTSUPP;
+
+                rtl8125_oob_mutex_lock(tp);
+                rtl8125_ocp_write(tp, my_cmd.offset, my_cmd.len, my_cmd.data);
+                rtl8125_oob_mutex_unlock(tp);
+                break;
+
+        case RTL_ENABLE_PCI_DIAG:
+                tp->rtk_enable_diag = 1;
+
+                dprintk("enable rtk diag\n");
                 break;
 
         case RTL_DISABLE_PCI_DIAG:
-                spin_lock_irqsave(&tp->lock, flags);
                 tp->rtk_enable_diag = 0;
-                spin_unlock_irqrestore(&tp->lock, flags);
 
-                printk("disable rtk diag\n");
+                dprintk("disable rtk diag\n");
                 break;
 
         case RTL_READ_MAC_OCP:
                 if (my_cmd.offset % 2)
                         return -EOPNOTSUPP;
 
-                spin_lock_irqsave(&tp->lock, flags);
-                my_cmd.data = rtl8101_mac_ocp_read(tp, my_cmd.offset);
-                spin_unlock_irqrestore(&tp->lock, flags);
-
+                my_cmd.data = rtl8125_mac_ocp_read(tp, my_cmd.offset);
                 if (copy_to_user(ifr->ifr_data, &my_cmd, sizeof(my_cmd))) {
                         ret = -EFAULT;
                         break;
@@ -216,16 +235,11 @@ int rtl8101_tool_ioctl(struct rtl8101_private *tp, struct ifreq *ifr)
                 if ((my_cmd.offset % 2) || (my_cmd.len != 2))
                         return -EOPNOTSUPP;
 
-                spin_lock_irqsave(&tp->lock, flags);
-                rtl8101_mac_ocp_write(tp, my_cmd.offset, (u16)my_cmd.data);
-                spin_unlock_irqrestore(&tp->lock, flags);
+                rtl8125_mac_ocp_write(tp, my_cmd.offset, (u16)my_cmd.data);
                 break;
 
         case RTL_DIRECT_READ_PHY_OCP:
-                spin_lock_irqsave(&tp->lock, flags);
-                my_cmd.data = rtl8101_mdio_prot_direct_read_phy_ocp(tp, my_cmd.offset);
-                spin_unlock_irqrestore(&tp->lock, flags);
-
+                my_cmd.data = rtl8125_mdio_prot_direct_read_phy_ocp(tp, my_cmd.offset);
                 if (copy_to_user(ifr->ifr_data, &my_cmd, sizeof(my_cmd))) {
                         ret = -EFAULT;
                         break;
@@ -234,9 +248,7 @@ int rtl8101_tool_ioctl(struct rtl8101_private *tp, struct ifreq *ifr)
                 break;
 
         case RTL_DIRECT_WRITE_PHY_OCP:
-                spin_lock_irqsave(&tp->lock, flags);
-                rtl8101_mdio_prot_direct_write_phy_ocp(tp, my_cmd.offset, my_cmd.data);
-                spin_unlock_irqrestore(&tp->lock, flags);
+                rtl8125_mdio_prot_direct_write_phy_ocp(tp, my_cmd.offset, my_cmd.data);
                 break;
 
         default:
